import 'reflect-metadata';
import * as i1 from '@kuoki/environment';
import { isPath, DefaultEnvironmentLoader as DefaultEnvironmentLoader$1, DefaultEnvironmentQuery as DefaultEnvironmentQuery$1, DefaultEnvironmentService as DefaultEnvironmentService$1, DefaultEnvironmentStore as DefaultEnvironmentStore$1, EnvironmentStore, EnvironmentService, EnvironmentQuery, EnvironmentLoader, prefixPath, pathAsString } from '@kuoki/environment';
import { CommonModule } from '@angular/common';
import * as i0 from '@angular/core';
import { InjectionToken, inject, Injectable, Optional, Inject, APP_INITIALIZER, Injector, NgModule, Pipe } from '@angular/core';
import { pickBy, get } from 'lodash-es';
import * as i1$1 from '@angular/router';
import { UrlTree } from '@angular/router';
import { of, filter, delay, merge, map } from 'rxjs';

/* eslint-disable @typescript-eslint/ban-types */
const ENVIRONMENT_PREFIX_METADATA_KEY = Symbol.for(`environment-prefix`);
/**
 * Sets a base path for the class used by the other decorators.
 * @param path The EnvironmentState path to resolve.
 * @returns A class decorator to set a base path for the class used by the other decorators.
 */
function EnvironmentPrefix(path) {
    return (target) => {
        if (isPath(path)) {
            Reflect.defineMetadata(ENVIRONMENT_PREFIX_METADATA_KEY, path, target);
        }
    };
}

/**
 * Check if the value is a Class.
 * @param value The value to check.
 * @returns `true` if the value is a Class, otherwise `false`.
 */
function isClass(value) {
    return typeof value === 'function' && 'prototype' in value && /^\s*class\s+/.test(value.toString());
}

/**
 * Creates a suitable Provider object using a config value and a default value.
 * @param value The provider values.
 * @returns A suitable Provider.
 */
function configProviderFactory(value) {
    var _a;
    const providerValue = (_a = value.configValue) !== null && _a !== void 0 ? _a : value.defaultValue;
    if (providerValue instanceof InjectionToken) {
        return { provide: value.provide, useExisting: providerValue, deps: value.deps, multi: value.multi };
    }
    if (isClass(providerValue)) {
        return { provide: value.provide, useClass: providerValue, deps: value.deps, multi: value.multi };
    }
    if (typeof providerValue === 'function') {
        return { provide: value.provide, useFactory: providerValue, deps: value.deps, multi: value.multi };
    }
    return { provide: value.provide, useValue: providerValue, deps: value.deps, multi: value.multi };
}

/**
 * A factory to create the GetOptionsAll object.
 * @param options the original GetOptionsAll.
 * @returns A safe GetOptionsAll.
 */
function getOptionsFactory(options) {
    if (options == null) {
        return {};
    }
    const getOptions = {
        defaultValue: options.defaultValue,
        targetType: options.targetType,
        transpile: options.transpile,
        config: options.config
    };
    if ('required' in options) {
        return defined(Object.assign(Object.assign({}, getOptions), { required: options.required }));
    }
    if ('dueTime' in options) {
        return defined(Object.assign(Object.assign({}, getOptions), { dueTime: options.dueTime }));
    }
    return defined(getOptions);
}
function defined(obj) {
    return pickBy(obj, (v) => v !== undefined);
}

/**
 * [[include:helpers/README.md]]
 * @module Helpers
 */

/**
 * The source or list of sources from which to get the environment properties.
 */
const ENVIRONMENT_SOURCES = new InjectionToken('ENVIRONMENT_SOURCES');

/**
 * The environment sources as single injected source or array of injected sources.
 */
const ENVIRONMENT_SOURCES_FACTORY = new InjectionToken('ENVIRONMENT_SOURCES_FACTORY', {
    factory: () => {
        const sources = inject(ENVIRONMENT_SOURCES, {
            optional: true
        });
        if (!Array.isArray(sources)) {
            return sources;
        }
        const sourceObjects = sources
            .map((source) => (isClass(source) ? inject(source) : source))
            .map((source) => source)
            .filter((source) => source != null);
        if (sourceObjects.length === 0) {
            return null;
        }
        return sourceObjects.length === 1 ? sourceObjects[0] : sourceObjects;
    }
});

/**
 * [[include:source/README.md]]
 * @module EnvironmentSource
 */

class DefaultEnvironmentLoader extends DefaultEnvironmentLoader$1 {
    constructor(service, sources) {
        super(service, sources);
        this.service = service;
        this.sources = sources;
    }
}
DefaultEnvironmentLoader.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.2.6", ngImport: i0, type: DefaultEnvironmentLoader, deps: [{ token: i1.EnvironmentService }, { token: ENVIRONMENT_SOURCES_FACTORY, optional: true }], target: i0.ɵɵFactoryTarget.Injectable });
DefaultEnvironmentLoader.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "15.2.6", ngImport: i0, type: DefaultEnvironmentLoader });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.2.6", ngImport: i0, type: DefaultEnvironmentLoader, decorators: [{
            type: Injectable
        }], ctorParameters: function () {
        return [{ type: i1.EnvironmentService }, { type: undefined, decorators: [{
                        type: Optional
                    }, {
                        type: Inject,
                        args: [ENVIRONMENT_SOURCES_FACTORY]
                    }] }];
    } });

/**
 * [[include:loader/README.md]]
 * @module EnvironmentLoader
 */

/**
 * Configuration parameters for the environment query.
 */
const ENVIRONMENT_QUERY_CONFIG = new InjectionToken('ENVIRONMENT_QUERY_CONFIG');

class DefaultEnvironmentQuery extends DefaultEnvironmentQuery$1 {
    constructor(store, queryConfig) {
        super(store, queryConfig);
        this.store = store;
        this.queryConfig = queryConfig;
    }
}
DefaultEnvironmentQuery.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.2.6", ngImport: i0, type: DefaultEnvironmentQuery, deps: [{ token: i1.EnvironmentStore }, { token: ENVIRONMENT_QUERY_CONFIG, optional: true }], target: i0.ɵɵFactoryTarget.Injectable });
DefaultEnvironmentQuery.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "15.2.6", ngImport: i0, type: DefaultEnvironmentQuery });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.2.6", ngImport: i0, type: DefaultEnvironmentQuery, decorators: [{
            type: Injectable
        }], ctorParameters: function () {
        return [{ type: i1.EnvironmentStore }, { type: undefined, decorators: [{
                        type: Optional
                    }, {
                        type: Inject,
                        args: [ENVIRONMENT_QUERY_CONFIG]
                    }] }];
    } });

/**
 * [[include:query/README.md]]
 * @module EnvironmentQuery
 */

class DefaultEnvironmentService extends DefaultEnvironmentService$1 {
    constructor(store) {
        super(store);
        this.store = store;
    }
}
DefaultEnvironmentService.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.2.6", ngImport: i0, type: DefaultEnvironmentService, deps: [{ token: i1.EnvironmentStore }], target: i0.ɵɵFactoryTarget.Injectable });
DefaultEnvironmentService.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "15.2.6", ngImport: i0, type: DefaultEnvironmentService });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.2.6", ngImport: i0, type: DefaultEnvironmentService, decorators: [{
            type: Injectable
        }], ctorParameters: function () { return [{ type: i1.EnvironmentStore }]; } });

/**
 * [[include:service/README.md]]
 * @module EnvironmentService
 */

/**
 * The initial state for the environment store.
 */
const ENVIRONMENT_INITIAL_STATE = new InjectionToken('ENVIRONMENT_INITIAL_STATE', {
    providedIn: 'root',
    factory: () => ({})
});

class DefaultEnvironmentStore extends DefaultEnvironmentStore$1 {
    constructor(initialState = {}) {
        super(initialState);
        this.initialState = initialState;
    }
}
DefaultEnvironmentStore.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.2.6", ngImport: i0, type: DefaultEnvironmentStore, deps: [{ token: ENVIRONMENT_INITIAL_STATE }], target: i0.ɵɵFactoryTarget.Injectable });
DefaultEnvironmentStore.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "15.2.6", ngImport: i0, type: DefaultEnvironmentStore });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.2.6", ngImport: i0, type: DefaultEnvironmentStore, decorators: [{
            type: Injectable
        }], ctorParameters: function () {
        return [{ type: undefined, decorators: [{
                        type: Inject,
                        args: [ENVIRONMENT_INITIAL_STATE]
                    }] }];
    } });

/**
 * [[include:store/README.md]]
 * @module EnvironmentStore
 */

function loadBeforeInitFactory(loader) {
    return () => loader.load();
}
class EnvironmentModule {
    /**
     * The singleton instance of EnvironmentQuery.
     * @throws If EnvironmentQuery is not provided.
     */
    static get query() {
        return EnvironmentModule._query;
    }
    /**
     * Initializes the EnvironmentModule with all the required services.
     * @param config Customizes the environment behavior and services.
     * @returns The EnvironmentModule with all the required services.
     */
    static forRoot(config) {
        var _a;
        return {
            ngModule: EnvironmentModule,
            providers: [
                configProviderFactory({
                    provide: ENVIRONMENT_INITIAL_STATE,
                    configValue: config === null || config === void 0 ? void 0 : config.initialState,
                    defaultValue: {}
                }),
                configProviderFactory({
                    provide: EnvironmentStore,
                    configValue: config === null || config === void 0 ? void 0 : config.store,
                    defaultValue: DefaultEnvironmentStore
                }),
                configProviderFactory({
                    provide: EnvironmentService,
                    configValue: config === null || config === void 0 ? void 0 : config.service,
                    defaultValue: DefaultEnvironmentService,
                    deps: [EnvironmentStore]
                }),
                configProviderFactory({
                    provide: ENVIRONMENT_QUERY_CONFIG,
                    configValue: config === null || config === void 0 ? void 0 : config.queryConfig,
                    defaultValue: null
                }),
                configProviderFactory({
                    provide: EnvironmentQuery,
                    configValue: config === null || config === void 0 ? void 0 : config.query,
                    defaultValue: DefaultEnvironmentQuery,
                    deps: [EnvironmentStore, ENVIRONMENT_QUERY_CONFIG]
                }),
                configProviderFactory({
                    provide: ENVIRONMENT_SOURCES,
                    configValue: config === null || config === void 0 ? void 0 : config.sources,
                    defaultValue: null
                }),
                configProviderFactory({
                    provide: EnvironmentLoader,
                    configValue: config === null || config === void 0 ? void 0 : config.loader,
                    defaultValue: DefaultEnvironmentLoader,
                    deps: [EnvironmentService, ENVIRONMENT_SOURCES_FACTORY]
                }),
                ((_a = config === null || config === void 0 ? void 0 : config.loadBeforeInit) !== null && _a !== void 0 ? _a : true)
                    ? {
                        provide: APP_INITIALIZER,
                        useFactory: loadBeforeInitFactory,
                        deps: [EnvironmentLoader],
                        multi: true
                    }
                    : []
            ]
        };
    }
    static forChild(config) {
        return {
            ngModule: EnvironmentModule,
            providers: [
                configProviderFactory({
                    provide: ENVIRONMENT_SOURCES,
                    configValue: config === null || config === void 0 ? void 0 : config.sources,
                    defaultValue: null
                }),
                configProviderFactory({
                    provide: EnvironmentLoader,
                    configValue: config === null || config === void 0 ? void 0 : config.loader,
                    defaultValue: DefaultEnvironmentLoader,
                    deps: [EnvironmentService, ENVIRONMENT_SOURCES_FACTORY]
                })
            ]
        };
    }
    constructor(injector) {
        this.injector = injector;
        const query = this.injector.get(EnvironmentQuery, null);
        if (query == null) {
            throw new Error('An instance of EnvironmentQuery is required. ' +
                'Use EnvironmentModule.forRoot() or provide the EnvironmentQuery service');
        }
        EnvironmentModule._query = query;
    }
}
EnvironmentModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.2.6", ngImport: i0, type: EnvironmentModule, deps: [{ token: Injector }], target: i0.ɵɵFactoryTarget.NgModule });
EnvironmentModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "15.2.6", ngImport: i0, type: EnvironmentModule, imports: [CommonModule] });
EnvironmentModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "15.2.6", ngImport: i0, type: EnvironmentModule, imports: [CommonModule] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.2.6", ngImport: i0, type: EnvironmentModule, decorators: [{
            type: NgModule,
            args: [{ imports: [CommonModule] }]
        }], ctorParameters: function () {
        return [{ type: i0.Injector, decorators: [{
                        type: Inject,
                        args: [Injector]
                    }] }];
    } });

/**
 * [[include:module/README.md]]
 * @module EnvironmentModule
 */

function environmentValueDecoratorFactory({ getEnvironmentValueFn, path, options }) {
    return (target, propertyKey, descriptor) => {
        if ((descriptor === null || descriptor === void 0 ? void 0 : descriptor.value) != null) {
            return descriptor;
        }
        const metadataKey = Symbol.for(`environment-value-decorator:${String(propertyKey)}`);
        const localDescriptor = descriptor != null
            ? descriptor
            : {
                enumerable: true,
                configurable: true
            };
        localDescriptor.get = function () {
            let value = Reflect.getMetadata(metadataKey, this);
            if (value === undefined) {
                const prefixDecorator = Reflect.getMetadata(ENVIRONMENT_PREFIX_METADATA_KEY, target.constructor);
                const localPath = prefixDecorator != null ? prefixPath(path, prefixDecorator) : path;
                const getOptions = getOptionsFactory(options);
                value = getEnvironmentValueFn(localPath, getOptions);
                if ((options === null || options === void 0 ? void 0 : options.static) !== false) {
                    Reflect.defineMetadata(metadataKey, value, this);
                }
            }
            return value;
        };
        localDescriptor.set = function (value) {
            Reflect.defineMetadata(metadataKey, value, this);
        };
        Object.defineProperty(target, propertyKey, localDescriptor);
    };
}

function EnvironmentValue(path, options) {
    return environmentValueDecoratorFactory({
        getEnvironmentValueFn: (_path, _options) => { var _a; return (_a = EnvironmentModule.query) === null || _a === void 0 ? void 0 : _a.get(_path, _options); },
        path,
        options
    });
}

function EnvironmentValueAsync(path, options) {
    return environmentValueDecoratorFactory({
        getEnvironmentValueFn: (_path, _options) => { var _a; return (_a = EnvironmentModule.query) === null || _a === void 0 ? void 0 : _a.getAsync(_path, _options); },
        path,
        options
    });
}

function EnvironmentValue$(path, options) {
    return environmentValueDecoratorFactory({
        getEnvironmentValueFn: (_path, _options) => { var _a; return (_a = EnvironmentModule.query) === null || _a === void 0 ? void 0 : _a.get$(_path, _options); },
        path,
        options
    });
}

/**
 * [[include:decorators/README.md]]
 * @module Decorators
 */

/**
 * Prevent path to be activated if required environment properties doesn't exist.
 */
class CanActivateEnvironment {
    constructor(router, query) {
        this.router = router;
        this.query = query;
        /**
         * The path to manage properties on route Data.
         */
        this.dataPath = 'canActivateEnvironment';
    }
    canActivate(route) {
        const properties = this.getProperties(route);
        if (!this.isNotEmptyProperties(properties)) {
            return of(true);
        }
        const dueTime = this.getDueTime(route);
        const urlTree = this.getUrlTree(route);
        const containsAll$ = this.query
            .containsAll$(...properties)
            .pipe(filter((value) => value === true));
        if (dueTime != null) {
            const timeout$ = of(urlTree !== null && urlTree !== void 0 ? urlTree : false).pipe(delay(dueTime));
            return merge(timeout$, containsAll$);
        }
        return containsAll$;
    }
    getProperties(route) {
        return get(route, `data.${this.dataPath}.properties`, this.properties);
    }
    getUrlTree(route) {
        const urlOnError = get(route, `data.${this.dataPath}.urlOnError`, this.urlOnError);
        if (typeof urlOnError === 'string') {
            return this.router.parseUrl(urlOnError);
        }
        return urlOnError instanceof UrlTree ? urlOnError : undefined;
    }
    isNotEmptyProperties(properties) {
        return properties != null && Array.isArray(properties) && properties.length > 0;
    }
    getDueTime(route) {
        const due = get(route, `data.${this.dataPath}.dueTime`);
        return typeof due === 'number' ? due : this.dueTime;
    }
}
CanActivateEnvironment.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.2.6", ngImport: i0, type: CanActivateEnvironment, deps: [{ token: i1$1.Router }, { token: i1.EnvironmentQuery }], target: i0.ɵɵFactoryTarget.Injectable });
CanActivateEnvironment.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "15.2.6", ngImport: i0, type: CanActivateEnvironment });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.2.6", ngImport: i0, type: CanActivateEnvironment, decorators: [{
            type: Injectable
        }], ctorParameters: function () { return [{ type: i1$1.Router }, { type: i1.EnvironmentQuery }]; } });

/**
 * [[include:guards/README.md]]
 * @module Guards
 */

/**
 * Renders the value from EnvironmentState or the path if value is undefined.
 */
class EnvironmentValuePipe {
    /**
     * Transforms the value from EnvironmentState or the path if value is undefined.
     * @template T The EnvironmentState property type.
     * @template K The expected property target type.
     * @param path The EnvironmentState path to resolve.
     * @param config The options to get a property.
     * @returns The value from EnvironmentState or the path if value is undefined.
     * @see EnvironmentQuery#get
     */
    transform(path, config) {
        var _a;
        const value = (_a = EnvironmentModule.query) === null || _a === void 0 ? void 0 : _a.get(path, config);
        return value !== undefined ? value : pathAsString(path);
    }
}
EnvironmentValuePipe.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.2.6", ngImport: i0, type: EnvironmentValuePipe, deps: [], target: i0.ɵɵFactoryTarget.Pipe });
EnvironmentValuePipe.ɵpipe = i0.ɵɵngDeclarePipe({ minVersion: "14.0.0", version: "15.2.6", ngImport: i0, type: EnvironmentValuePipe, name: "envValue" });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.2.6", ngImport: i0, type: EnvironmentValuePipe, decorators: [{
            type: Pipe,
            args: [{ name: 'envValue' }]
        }] });

/**
 * Renders the value as Observable from EnvironmentState or the path as Observable if value is undefined.
 */
class EnvironmentValueReactivePipe {
    /**
     * Transforms the value as Observable from EnvironmentState or the path as Observable if value is undefined.
     * @template T The EnvironmentState property type.
     * @template K The expected property target type.
     * @param path The EnvironmentState path to resolve.
     * @param config The options to get a property as Observable.
     * @returns The value as Observable from EnvironmentState or the path as Observable if value is undefined.
     * @see EnvironmentQuery#get$
     */
    transform(path, config) {
        var _a, _b;
        return ((_b = (_a = EnvironmentModule.query) === null || _a === void 0 ? void 0 : _a.get$(path, config).pipe(map((value) => (value !== undefined ? value : pathAsString(path))))) !== null && _b !== void 0 ? _b : of(pathAsString(path)));
    }
}
EnvironmentValueReactivePipe.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.2.6", ngImport: i0, type: EnvironmentValueReactivePipe, deps: [], target: i0.ɵɵFactoryTarget.Pipe });
EnvironmentValueReactivePipe.ɵpipe = i0.ɵɵngDeclarePipe({ minVersion: "14.0.0", version: "15.2.6", ngImport: i0, type: EnvironmentValueReactivePipe, name: "environmentValue$" });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.2.6", ngImport: i0, type: EnvironmentValueReactivePipe, decorators: [{
            type: Pipe,
            args: [{ name: 'environmentValue$' }]
        }] });

/**
 * [[include:pipes/README.md]]
 * @module Pipes
 */

/**
 * @module EnvironmentAngular
 */

/**
 * Generated bundle index. Do not edit.
 */

export { CanActivateEnvironment, DefaultEnvironmentLoader, DefaultEnvironmentQuery, DefaultEnvironmentService, DefaultEnvironmentStore, ENVIRONMENT_INITIAL_STATE, ENVIRONMENT_PREFIX_METADATA_KEY, ENVIRONMENT_QUERY_CONFIG, ENVIRONMENT_SOURCES, ENVIRONMENT_SOURCES_FACTORY, EnvironmentModule, EnvironmentPrefix, EnvironmentValue, EnvironmentValue$, EnvironmentValueAsync, EnvironmentValuePipe, EnvironmentValueReactivePipe, configProviderFactory, environmentValueDecoratorFactory, getOptionsFactory, isClass };
//# sourceMappingURL=kuoki-environment-angular.mjs.map
